@@ -7,6 +7,7 @@ from datetime import datetime, timedelta
 from enum import Enum
 from functools import wraps
 from logging import DEBUG, getLogger
+from textwrap import dedent
 from threading import Thread
 from time import sleep, strptime
 from typing import (
@@ -29,8 +30,7 @@ from async_upnp_client.client import UpnpDevice, UpnpService, UpnpStateVariable
 from async_upnp_client.client_factory import UpnpFactory
 from async_upnp_client.exceptions import UpnpActionResponseError, UpnpResponseError
 from async_upnp_client.utils import get_local_ip
-from mypy_extensions import Arg
-from pydantic import BaseModel, Extra, Field  # pylint: disable=no-name-in-module
+from pydantic import BaseModel, Extra, Field
 from xmltodict import parse as parse_xml
 
 from wg_utilities.functions import traverse_dict
@@ -468,7 +468,7 @@ class Yas209Service(Enum):
 
 def _needs_device(
     func: Callable[[YamahaYas209], Coroutine[Any, Any, Union[Mapping[str, Any], None]]]
-) -> Callable[[Arg(YamahaYas209, "yas_209")], Any]:
+) -> Callable[[YamahaYas209], Any]:
     """This decorator is used when the DLNA device is needed and provides a clean
      way of instantiating it lazily
 
@@ -538,6 +538,9 @@ class YamahaYas209:
         on_track_update: Optional[Callable[[CurrentTrack.Info], None]] = None,
         on_state_update: Optional[Callable[[str], None]] = None,
         logging: bool = True,
+        listen_ip: Optional[str] = None,
+        listen_port: Optional[int] = None,
+        source_port: Optional[int] = None,
     ):
         self.ip = ip
         self.on_event = on_event
@@ -558,6 +561,15 @@ class YamahaYas209:
         self.device: UpnpDevice
 
         self._logging = logging
+        self._listen_ip = listen_ip
+        self._listen_port = listen_port
+        self._source_port = source_port or 0
+
+        if self._listen_ip is not None and self._listen_port is None:
+            raise TypeError(
+                "Argument `listen_port` cannot be None when `listen_ip` is not None:"
+                f" {self._listen_ip!r}"
+            )
 
         if start_listener:
             self.listen()
@@ -568,15 +580,24 @@ class YamahaYas209:
         if self._listening:
             return
 
+        worker_exception: Optional[Exception] = None
+
         def _worker() -> None:
-            new_event_loop().run_until_complete(self._subscribe())
+            nonlocal worker_exception
+            try:
+                new_event_loop().run_until_complete(self._subscribe())
+            except Exception as exc:  # pylint: disable=broad-except
+                worker_exception = exc
 
         listener_thread = Thread(target=_worker)
         listener_thread.start()
 
-        while not self._listening:
+        while not self._listening and worker_exception is None:
             sleep(0.01)
 
+        if worker_exception is not None:
+            raise worker_exception  # pylint: disable=raising-bad-type
+
     def on_event_wrapper(
         self, service: UpnpService, service_variables: Sequence[UpnpStateVariable[str]]
     ) -> None:
@@ -633,10 +654,40 @@ class YamahaYas209:
         """Subscribe to service(s) and output updates."""
 
         # start notify server/event handler
-        source = (get_local_ip(self.device.device_url), 0)
-        server = AiohttpNotifyServer(self.device.requester, source=source)
+        local_ip = get_local_ip(self.device.device_url)
+        source = (local_ip, self._source_port)
+
+        callback_url = (
+            None
+            if self._listen_port is None
+            else f"http://{self._listen_ip or local_ip}:{self._listen_port}/notify"
+        )
+        server = AiohttpNotifyServer(
+            self.device.requester, source=source, callback_url=callback_url
+        )
         await server.async_start_server()
-        event_handler = server.event_handler
+
+        if self._logging:
+            LOGGER.debug(
+                dedent(
+                    """
+            Listen IP:          %s
+            Listen Port:        %s
+            Source IP:          %s
+            Source Port:        %s
+            Callback URL:       %s
+            Server Listen IP:   %s
+            Server Listen Port: %s
+            """
+                ),
+                self._listen_ip,
+                str(self._listen_port),
+                local_ip,
+                str(self._source_port),
+                str(callback_url),
+                server.listen_ip,
+                server.listen_port,
+            )
 
         # create service subscriptions
         services = []
@@ -648,7 +699,8 @@ class YamahaYas209:
             service.on_event = self.on_event_wrapper
             services.append(service)
             try:
-                await event_handler.async_subscribe(service)
+                await server.event_handler.async_subscribe(service)
+                LOGGER.debug("Subscribed to %s", service.service_id)
             except UpnpResponseError as exc:
                 if self._logging:
                     LOGGER.exception(
@@ -667,7 +719,9 @@ class YamahaYas209:
                 if self._listening is False:
                     return
                 await async_sleep(1)
-            await event_handler.async_resubscribe_all()
+
+            LOGGER.debug("Resubscribing to all services")
+            await server.event_handler.async_resubscribe_all()
 
             services_to_remove = []
             for service in failed_subscriptions:
@@ -678,7 +732,7 @@ class YamahaYas209:
                             " %s",
                             service.service_id,
                         )
-                    await event_handler.async_subscribe(service)
+                    await server.event_handler.async_subscribe(service)
                     services_to_remove.append(service)
                 except UpnpResponseError as exc:
                     if self._logging:
@@ -701,7 +755,7 @@ class YamahaYas209:
         action: str,
         callback: Optional[Callable[[Mapping[str, Any]], Any]] = None,
         **call_kwargs: Union[str, int],
-    ) -> Dict[str, Any]:
+    ) -> Union[Dict[str, Any], None]:
 
         if action not in service.actions:
             raise ValueError(
@@ -728,14 +782,14 @@ class YamahaYas209:
                 f" {', '.join(service.actions)}"
             )
         try:
-            return run(_worker(self))
+            return run(_worker(self))  # type: ignore[no-any-return]
         except UpnpActionResponseError:
             # if not trying to transition to the current state (and current state is
             # known)
             if self.state != Yas209State.UNKNOWN and self.state.action != action:
                 raise
 
-            return {}
+            return None
 
     @staticmethod
     def _parse_xml_dict(xml_dict: Dict[str, Any]) -> None:
@@ -931,8 +985,9 @@ class YamahaYas209:
             dict: the response in JSON form
         """
 
-        media_info = self._call_service_action(
-            Yas209Service.AVT, "GetMediaInfo", InstanceID=0
+        media_info = (
+            self._call_service_action(Yas209Service.AVT, "GetMediaInfo", InstanceID=0)
+            or {}
         )
 
         self._parse_xml_dict(media_info)
@@ -969,13 +1024,16 @@ class YamahaYas209:
         """
         if not hasattr(self, "_volume_level"):
 
-            res = self._call_service_action(
-                Yas209Service.RC,
-                "GetVolume",
-                InstanceID=0,
-                Channel="Master",
+            res = (
+                self._call_service_action(
+                    Yas209Service.RC,
+                    "GetVolume",
+                    InstanceID=0,
+                    Channel="Master",
+                )
+                or {}
             )
 
-            self._volume_level = float(res["CurrentVolume"] / 100)
+            self._volume_level = float(res.get("CurrentVolume", 0) / 100)
 
         return self._volume_level
