@@ -1,8 +1,9 @@
 """Functions for managing processes"""
 
-from logging import getLogger, DEBUG
+from logging import DEBUG, getLogger
 from re import compile as compile_regex
-from subprocess import Popen, PIPE
+from subprocess import PIPE, Popen
+from typing import Tuple
 
 from wg_utilities.loggers import add_stream_handler
 
@@ -13,7 +14,9 @@ add_stream_handler(LOGGER)
 COMMAND_PATTERN = compile_regex(r"""((?:[^\s"']|"[^"]*"|'[^']*')+)""")
 
 
-def run_cmd(cmd, exit_on_error=True, shell=False):
+def run_cmd(
+    cmd: str, exit_on_error: bool = True, shell: bool = False
+) -> Tuple[str, str]:
     """Helper function for running commands on the command line
 
     Args:
@@ -36,12 +39,12 @@ def run_cmd(cmd, exit_on_error=True, shell=False):
     with Popen(popen_input, stdout=PIPE, stderr=PIPE, shell=shell) as process:
         output, error = process.communicate()
 
-        error = error.decode("utf-8").strip()
+        error_str = error.decode("utf-8").strip()
 
         if process.returncode != 0:
             if exit_on_error:
-                raise RuntimeError(error)
+                raise RuntimeError(error_str)
 
-            LOGGER.error(error)
+            LOGGER.error(error_str)
 
-    return output.decode("utf-8").strip(), error
+    return output.decode("utf-8").strip(), error_str
